#include "queue.h"

struct Queue queue_create(unsigned int max_size) {
  struct Queue q;
  q.buff = malloc(sizeof(char) * (max_size + 1));
  q.max_size = max_size + 1;
  q.start = max_size;
  q.end = 0;
  return q;
}

int queue_init(struct Queue *q, unsigned int max_size) {
  q->buff = malloc(sizeof(unsigned char) * (max_size + 1));
  if (q->buff == NULL) return -1;
  q->max_size = max_size + 1; // required for start/end pointers
  q->start = max_size;
  q->end = 0;
  return 0;
}

void queue_destroy(struct Queue *q) {
  free(q->buff);
}

int queue_add(volatile struct Queue *q, unsigned char c) {
  if (queue_full(q)) {
    return -1;
  }
  q->buff[q->end] = c;
  q->end += 1;
  q->end %= q->max_size;
  return 0;
}

int queue_remove(volatile struct Queue *q, unsigned char *c) {
  if (queue_empty(q)) {
    return -1;
  }
  q->start += 1;
  q->start %= q->max_size;
  *c = q->buff[q->start];
  return 0;
}

int queue_size(volatile struct Queue *q) {
  return (q->max_size + q->end - q->start - 1) % q->max_size;
}

int queue_full(volatile struct Queue *q) {
  return q->start == q->end;;
}

int queue_empty(volatile struct Queue *q) {
  return queue_size(q) == 0;
}

void queue_print(struct Queue *q) {
  int i;
  printf("buffer contents:\n");
  for (i = 0; i < q->max_size; i += 1) {
    char c = (char) q->buff[i];
    if (c == 0) c = '-';
    printf("%c", c);;
    if (i == q->start) printf("  <- start");
    if (i == q->end) printf("  <- end");
    printf("\n");
  }
}
